/*
    This program uses a shared variable to implement
    a simple synchronization pattern between two threads.
    The synchronization pattern is to keep the two threads
    "alternating" with each other. Each thread does a sequence
    of "computations," where each computation can take a
    different amount of time, and the threads are synchronized
    so that each thread begins a computation exactly when the
    other threads ends a computation.
*/
#include <windows.h>  // Sleep(), Win32
#include <unistd.h>   // sleep(), Posix
#include <stdio.h>    // fprintf(), stderr
#include <stdlib.h>   // exit(), NULL
#include <pthread.h>

#define RUN_TIME 30  // run time in seconds

// global variables
int runFlag = 1;

int sharedVariable;

int main()
{
   void *thread1(void *vargp);  //thread function prototypes
   void *thread2(void *vargp);

   int threadParam1 = 0;       // thread function parameters
   int threadParam2 = 0;

   pthread_t tid1;
   pthread_t tid2;

   // initialize the shared variable
   sharedVariable = 1;

   // create two threads
   printf("alternating: Starting threads\n");
   pthread_create(&tid1, NULL, thread1, &threadParam1);
   pthread_create(&tid2, NULL, thread2, &threadParam2);

   //sleep(RUN_TIME); // let the threads run for a while
   Sleep(1000*RUN_TIME);
   runFlag = 0;     // now stop the threads
   //sleep(1);      // give the threads time to stop
   Sleep(1000);
   
   printf("\nThreads stopped.\n");

   return 0;
}//main


void *thread1(void *vargp)
{
   int i;
   int workTime;
   srand(1);      // seed the random number generator
   while(runFlag)
   {
      while( sharedVariable == 0 )
      {
         //sleep(0);
         Sleep(0);
      }
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "\nThread1 working for %d seconds", workTime);
      for (i = 0; i < workTime; i++)
      {
         fprintf(stderr, ".");
         //sleep(1);
         Sleep(1000);
      }
      fprintf(stderr, "done");
      sharedVariable = 0;
   }
   return NULL;
}//thread1


void *thread2(void *vargp)
{
   int i;
   int workTime;
   srand(2);      // seed the random number generator
   while(runFlag)
   {
      while( sharedVariable == 1 )
      {
         //sleep(0);
         Sleep(0);
      }
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "\nThread2 working for %d seconds", workTime);
      for (i = 0; i < workTime; i++)
      {
         fprintf(stderr, ".");
         //sleep(1);
         Sleep(1000);
      }
      fprintf(stderr, "done");
      sharedVariable = 1;
   }
   return NULL;
}//thread2
