/*
    This program uses two binary semaphores to implement
    a simple synchronization pattern between two threads.
    The synchronization pattern is to keep the two threads
    "alternating" with each other. Each thread does a sequence
    of "computations," where each computation can take a
    different amount of time, and the threads are synchronized
    so that each thread begins a computation exactly when the
    other threads ends a computation.

    Notice that this program has the same pattern of waits and
    posts as the program lockstep2.c, the only difference being
    in the way the semaphores are initialized.

    Notice that with this pattern, there can be a gap in time
    between when thread1 finishes its first calculation and
    thread2 starts its first claculation (if thread2 is started
    some time after thread1). This doesn't happen with alternating1.c (why?).

    The wait-post pattern in this program is
             thread1    thread2
             -------    -------
              wait       wait
              work       work
              post       post
*/
#include <windows.h>  // Sleep(), Win32
#include <unistd.h>    // sleep(), Posix
#include <stdio.h>     // fprintf(), stderr
#include <stdlib.h>    // exit(), NULL
#include <pthread.h>
#include <semaphore.h>

#define RUN_TIME 30  // run time in seconds

// global variables
int runFlag = 1;

sem_t semaphore1;
sem_t semaphore2;

int main()
{
   void *thread1(void *vargp);  //thread function prototypes
   void *thread2(void *vargp);

   int threadParam1 = 0;       // thread function parameters
   int threadParam2 = 0;

   pthread_t tid1;
   pthread_t tid2;

   // initialize the (binary) semaphore
   sem_init(&semaphore1, 0, 1); // signaled
   sem_init(&semaphore2, 0, 0); // not signaled

   // create two threads
   printf("alternating: Starting threads\n");
   pthread_create(&tid1, NULL, thread1, &threadParam1);
   pthread_create(&tid2, NULL, thread2, &threadParam2);

   Sleep(1000*RUN_TIME);
   runFlag = 0;     // now stop the threads
   //sleep(1);        // give the threads time to stop
   Sleep(1000);
   
   printf("\nThreads stopped.\n");

   return 0;
}//main


void *thread1(void *vargp)
{
   int i;
   int workTime;
   srand(1);      // seed the random number generator
   while(runFlag)
   {
      sem_wait(&semaphore1);
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "\nThread1 working for %d seconds", workTime);
      for (i = 0; i < workTime; i++)
      {
         fprintf(stderr, ".");
         //sleep(1);
         Sleep(1000);
      }
      fprintf(stderr, "done");
      sem_post(&semaphore2);
   }
   return NULL;
}//thread1


void *thread2(void *vargp)
{
   int i;
   int workTime;
   srand(2);      // seed the random number generator
   while(runFlag)
   {
      sem_wait(&semaphore2);
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "\nThread2 working for %d seconds", workTime);
      for (i = 0; i < workTime; i++)
      {
         fprintf(stderr, ".");
         //sleep(1);
         Sleep(1000);
      }
      fprintf(stderr, "done");
      sem_post(&semaphore1);
   }
   return NULL;
}//thread2
