/*
    This program uses two shared variables to implement
    a simple synchronization pattern between two threads.
    The synchronization pattern is to keep the two threads in
    "lockstep" with each other. Each thread does a sequence
    of "computations," where each computation can take a
    different amount of time, and the threads are synchronized
    so that they begin each successive computation at the exact
    same time.

    This program seems to show that this simple synchronization
    pattern does not really need to be implemented using semaphores,
    just two shared memomory locations seems to be enough.
*/
#include <windows.h>   // Sleep(), Win32
#include <unistd.h>    // sleep(), Posix
#include <stdio.h>     // fprintf(), stderr
#include <stdlib.h>    // exit(), NULL
#include <pthread.h>

#define RUN_TIME 30  // run time in seconds

// global variables
int runFlag = 1;

int sharedVariable1;
int sharedVariable2;

int main()
{
   void *thread1(void *vargp);  //thread function prototypes
   void *thread2(void *vargp);

   int threadParam1 = 0;       // thread function parameters
   int threadParam2 = 0;

   pthread_t tid1;
   pthread_t tid2;

   // initialize the shared variables
   sharedVariable1 = 0;
   sharedVariable2 = 0;

   // create two threads
   printf("lockstep: Starting threads\n");
   pthread_create(&tid1, NULL, thread1, &threadParam1);
   pthread_create(&tid2, NULL, thread2, &threadParam2);

   //sleep(RUN_TIME); // let the threads run for a while
   Sleep(1000*RUN_TIME);
   runFlag = 0;     // now stop the threads
   //sleep(1);      // give the threads time to stop
   Sleep(1000);

   printf("\nThreads stopped.\n");

   return 0;
}//main


void *thread1(void *vargp)
{
   int workTime;
   srand(100);      // seed the random number generator
   while(runFlag)
   {
      sharedVariable2 = 1;
      while( sharedVariable1 == 0 )
      {
         //sleep(0);
         Sleep(0);
      }
      sharedVariable1 = 0;
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "Thread1 working for %d seconds\n", workTime);
      //sleep(workTime);
      Sleep(1000*workTime);
      fprintf(stderr, "--done1\n");
   }
   return NULL;
}//thread1


void *thread2(void *vargp)
{
   int workTime;
   srand(101);     // seed the random number generator
   while(runFlag)
   {
      sharedVariable1 = 1;
      while( sharedVariable2 == 0 )
      {
         //sleep(0);
         Sleep(0);
      }
      sharedVariable2 = 0;
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "Thread2 working for %d seconds\n", workTime);
      //sleep(workTime);
      Sleep(1000*workTime);
      fprintf(stderr, "--done2\n");
   }
   return NULL;
}//thread2
