/*
    This program implements a simple synchronization pattern
    between two threads. The synchronization pattern is to keep
    the two threads "alternating" with each other. Each thread
    does a sequence of "computations," where each computation can
    take a different amount of time, and the threads are synchronized
    so that each thread begins a computation exactly when the
    other threads ends a computation. In addition, whichever thread
    is ready to run first becomes the "first" thread in the alternating
    pattern. In other words, this program does not favor thread1 or
    thread2 as the prefered "first" thread.

    This program uses a mutex, two shared variables, and two local variables
    to implement the synchronization pattern.
*/
#include <stdio.h>
#include <windows.h>

#define RUN_TIME 30  // run time in seconds

// global variables
int runFlag = TRUE;

int commonFlag = 0;  // used to flag which thread is "first"
int flag1 = -1;
int flag2 = -1;
int turn = 0;        // used to switch between the two threads

int main( int argc, char *argv[])
{
   // thread function prototypes
   DWORD WINAPI thread1(LPVOID);
   DWORD WINAPI thread2(LPVOID);

   // create two threads
   printf("alternating1: Starting threads\n");
   CreateThread(NULL, 0, thread1, NULL, 0, NULL);
   CreateThread(NULL, 0, thread2, NULL, 0, NULL);

   Sleep(RUN_TIME*1000); // let the threads run for a while
   runFlag = FALSE;      // now stop the threads
   Sleep(200);           // give the threads time to stop

   printf("\nThreads stopped\n");

   return 0;
}//main


DWORD WINAPI thread1(LPVOID threadParam)
{
   int i;
   int workTime, delayTime;

   srand(GetTickCount());  // seed the random number generator

   delayTime = 1 + rand()%9;  // 1 to 9 seconds of delay
   fprintf(stderr, "Thread1 delayed for %d seconds\n", delayTime);
//   Sleep(delayTime*1000);

   // enter critical section
   flag1 = commonFlag;    // thread that gets value 0 is "first"
   Sleep(0);
   commonFlag++;          // we're assuming this is atomic (it is on a Pentium)
   // exit critical section

   if ( flag1 == flag2 )  // there was a race condition
      flag1 = 0;          // so compensate for the race condition
                          // (the above two lines are not needed in thread1???)
   while(runFlag)
   {
      while( turn != flag1 )  // wait for turn
         Sleep(0);
      workTime = 1 + rand()%9;          // 1 to 9 seconds of work time
      fprintf(stderr, "\nThread1 working for %d seconds", workTime);
      for (i = 0; i < workTime; i++)
      {
         fprintf(stderr, ".");
         Sleep(1000);
      }
      fprintf(stderr, "done");
      turn = 1 - turn;  // toggle 0 to 1 and 1 to 0
   }
   return 0;
}//thread1


DWORD WINAPI thread2(LPVOID threadParam)
{
   int i;
   int workTime, delayTime;

   srand(GetTickCount());  // seed the random number generator

   delayTime = 1 + rand()%9;  // 1 to 9 seconds of delay
   fprintf(stderr, "Thread2 delayed for %d seconds\n", delayTime);
//   Sleep(delayTime*1000);

   // enter critical section
   flag2 = commonFlag;    // thread that gets value 0 is "first"
   Sleep(0);
   commonFlag++;          // we're assuming this is atomic (it is on a Pentium)
   // exit critical section

   if ( flag1 == flag2 )  // there was a race condition
      flag2 = 1;          // so compensate for the race condition

   while(runFlag)
   {
      while( turn != flag2 )  // wait for turn
         Sleep(0);
      workTime = 1 + rand()%9;          // 1 to 9 seconds of work time
      fprintf(stderr, "\nThread2 working for %d seconds", workTime);
      for (i = 0; i < workTime; i++)
      {
         fprintf(stderr, ".");
         Sleep(1000);
      }
      fprintf(stderr, "done");
      turn = 1 - turn;  // toggle 0 to 1 and 1 to 0
   }
   return 0;
}//thread2
